@@ -5,38 +5,65 @@ class API(object):
         self.__uri = uri
         self.__verify = verify
         self.__db = {}
+        self.__plurals = {}
 
     @property
     def uri(self):
         return self.__uri
 
+    def clearCache(self):
+        self.__db = {}
+
     def get(self, resource):
         return requests.get("%s/%s"%(self.__uri,resource), verify=self.__verify)
 
     def post(self, resource, value):
         return requests.post("%s/%s"%(self.__uri, resource), data=value, verify=self.__verify)
 
-    def object(self, klass, id):
-        if klass not in self.__db:
-            self.__db[klass] = {}
-        return self.__db[klass][id]
+    def storeObject(self, obj):
+        if obj.uriType not in self.__db:
+            self.__db[obj.uriType] = {}
+        self.__db[obj.uriType][obj.id] = obj
+        self.__plurals[obj.type] = obj.uriType
 
-    def storeObject(self, klass, id, obj):
-        if klass not in self.__db:
-            self.__db[klass] = {}
-        self.__db[klass][id] = obj
+    def _expand(self, data):
+        ret = data
+        if (isinstance(data, dict)):
+          ret = {}
+          for attr in data.iterkeys():
+            ret[attr] = self._expand(data[attr])
+          if '_type' in data:
+            ret = ModelObject.new(self, ret)
+        elif (isinstance(data, list)):
+          ret = []
+          for d in data:
+            ret.append(self._expand(d))
+        elif (isinstance(data, basestring) and data.startswith("#")):
+          ret = ModelObject.new(self, data)
+        return ret
 
     def objects(self, uriType):
         ret = []
-        for res in self.get("%s/.json"%(uriType)).json:
-            obj = ModelObject.new(self, res)
-            ret.append(obj)
-        return ret
+        data = self.get("%s/.json"%(uriType)).json()
+        return self._expand(data)
 
-    def object(self, uriType, id):
-        res = self.get("%s/%s.json"%(uriType, id)).json
+    def object(self, uriType, id, refresh=False):
+        id = int(id)
+        if uriType not in self.__db:
+            self.__db[uriType] = {}
+        if id in self.__db[uriType]:
+            if refresh:
+                del self.__db[uriType][id]
+            else:
+                return self.__db[uriType][id]
+
+        res = self.get("%s/%s.json"%(uriType, id)).json()
         return ModelObject.new(self, res)
 
+    def resolve(self, singularType, id):
+        pluralType = self.__plurals[singularType]
+        return self.object(pluralType, id)
+
     def resources(self):
         return self.objects("resources")
 
@@ -59,7 +86,7 @@ class ModelObject(dict):
         self.__data = data
         self.__api = api
         if isinstance(self.__data, dict):
-            self.__api.storeObject(self.__data['_type'], self.__data['id'], self)
+            self.__api.storeObject(self)
 
     @property
     def api(self):
@@ -78,20 +105,19 @@ class ModelObject(dict):
         return self.resolve()['_plural_type']
 
     def qrCode(self, size=10):
-        return requests.get("%s%s/%s/qr-%s.png"%(self.__api.uri, 'resources', self.__data['id'], size), verify=False).content
+        return requests.get("%s%s/%s/qr-%s.png"%(self.__api.uri, 'resources', self.id, size), verify=False).content
 
     def refresh(self):
         if isinstance(self.__data, dict):
-            self.__data = self.api.object(self.uriType, self.id).__data
+            self.__data = self.api.object(self.uriType, self.id, refresh=True).__data
         else:
             self.resolve().refresh()
 
     def resolve(self):
         if isinstance(self.__data, dict):
             return self.__data
-        print "Resolving", self, self.__data
         tokens = self.__data.split("#")
-        return self.__api.object(tokens[1], tokens[2]).resolve()
+        return self.__api.resolve(tokens[1], tokens[2]).resolve()
 
     def keys(self):
         ret = []
@@ -100,12 +126,7 @@ class ModelObject(dict):
         return ret
 
     def __getitem__(self, key):
-        val = self.resolve()[key]
-        if isinstance(val, str):
-            tokens = val.split("#")
-            if len(tokens) == 3:
-                return self.__api.object(tokens[1], tokens[2])
-        return val
+        return self.resolve()[key]
 
     @staticmethod
     def new(api, data):
@@ -116,7 +137,6 @@ class ModelObject(dict):
             elif data['_type'] == "sensor":
                 cls = Sensor
         return cls(api, data)
-        
 
 class Sensor(ModelObject):
     def setValue(self, value):
